/*
DETAILS ON FITTING BINOMIAL, BETA-BINOMIAL, AND OTHER DISTRIBUTIONS TO
DISCRETE DATA.

Demonstration of the use of NLMIXED to fit the binomial and beta-binomal
to discrete data using maximum likelihood. Data files can contain just 
the counts (numbers) per sampling unit, labeled y here (not x), or both y 
and n (observations per sampling unit). Total number of sampling units
is N (where lower-case n is individuals in each sampling unit).

Program now also fits the logistic-normal-binomial (LNB or L-N-B) distribution 
to the data. This is another mixture distribution, with the mixing on the 
logit scale (with a normal distribution). x is conditionally binomial 
(conditional on the between-site random effect). 

Program also fits the CLL-normal-binomial (CLL-N-B) distribution. Same as
LNB, but with CLL link. Not done for all examples. 

If n is constant across sampling units, one can simply define it in a 
data step, such as: "n = 9;". See first example.

Generally, data are read as the counts per sampling unit, with a record
for each unit. However, one can also use "REPLICATE" to read in
frequencies. 

Parameters for LNB distribution are:
	logit ('linear predictor' or lp), and not p directly; and
	theta (the).
This makes it easy to expand for the inclusion of treatment effects, 
in the framework of generalized linear modeling of data.  For the 
beta-binomial, p is determined from the logit, and also the alpha 
and beta parameters are also displayed (based on p and theta), because
some references use the alpha/beta version of the distribution.
Moreover, the intraclass correlation is also calculated based
on estimated parameters. For the beta-binomial, the theta estimate 
is given under "Parameter Estimates". However, p (and alpha, beta,
& intraclass correlation) are given under "Additional Estimates".

The program uses a dual quasi-Newton optimization algorithm 
by default, which is different from the Newton-Raphson algorithm in 
the BBD program. Results could differ slightly. With NLMIXED, the
optimization algorithm can be changed using "tech=xxxxx" option.

The chi-square goodness of fit test is not done here based on observed
and expected frequencies. There is a separate SAS program for calculating 
a Pearson Goodness of Fit test, based on the Observed and Expected 
frequenicies in each class. This is in the same zip file.

	Program: LNB_BBD_Bin (GOF).SAS (written by Alissa Kriss).

In the current file, a likelihood-ratio test can
be conducted based on the log-likelihoods for the binomial and
beta-binomial (i.e., does beta-binomial fit better than binomial?).
In the "ancient* BBD program, only the portion of the log-likelihood 
involving the parameters is displayed. In NLMIXED, the full log-
likelihood is given. The differences in log-likelihoods (binomial vs.
beta-binomial) are the same for both programs.


Code by: L. V. Madden (Ohio State University). This code is supplied
as a courtesy for researchers. There is no guarantee that the 
program will work correctly in all situations.

For more information, contact him at MADDEN.1@osu.edu

*/


*---(1) Tobacco virus data (G1-11);
data tobacco;
input y @@;	*<--Read in data strung out across records (N=75 SUs total here);
n=40;		*<--When n (obs./SU) is always same, one can define it here;
subject=_n_;
datalines;
 	1    1    1    1    2    2    2    2    2    2    2    2    2    3    3
    3    3    3    3    3    3    4    4    4    4    4    4    4    4    4
    4    4    5    5    5    6    6    6    6    6    6    6    6    6    7
    7    7    7    7    7    7    7    8    8    8    8    8    8    9   10
   10   10   10   10   10   11   11   11   11   12   12   13   14   15   25
;

title 'Fitting distrete distributions to G1-11 tobacco virus example data, ex. 1 (G1-11)';
proc nlmixed data=tobacco ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
							*--Give initial guess of parameters;
    parameters lp = -2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
	
run;

*--The fitting of the beta-binomial using MLE (mixture distributio: mixing on p);
proc nlmixed data=tobacco ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--_Following lines give the log-linear formulation for the BBD log-likelihood;
	*--_The combinatorial and the Beta functions are all re-expresed as 
		log-gamma (lgamma) functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  		*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;

*---Now consider a different generalization of binomial (mixing on logits, not on p);
proc nlmixed data=tobacco ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'L-N-B distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 gam = 0.1 ; 	*--LOGIT linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    p=1/(1+exp(-eta));					*--Get p after first getting the logit;
    model y ~ binomial(n,p);			*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1/(1+exp(-lp));
	estimate 'gam^2' gam*gam;
	estimate 'pi' 1/(1+exp(-lp));
	estimate 'rho_' (gam*gam) / ( (gam*gam) + ((3.1415**2)/3) );
	estimate 'rho_2' (gam*gam)*pi*(1-pi);
	estimate 'rho_3' ((gam*gam)*(pi*(1-pi))**2)/( (gam*gam)*(pi*(1-pi))**2 + pi*(1-pi)) ;
run;

/*
Because the random effect is on a logit scale, there is no exact equation for the
intra-class correlation (rho) of binary disease status within sites (for L-N-B dist.). 
Several different approximations are in use (based on different assumptions). 
I think rho_2 or rho_3 may be best (based on Taylor series expansion, ...).
Rho_ is based on very different assumptions (based on a latent logistic distribution).
We provide code for different intra-class correlations in this program without
any strong recommendations for the choices. Other choices later. 
See Dissertation appendix in  A. Kriss (Ohio State) for some discussion on this 
topic for the CLL-N-B distribution, and refereneces therein. 
See also discussion on rho in Kriss et al. (2012). Phytopathology 102: 867-877.  

*/

proc glimmix data=tobacco method=laplace;	*<--with LAPLACE, get almost same results
												as with NLMIXED code;
title2 'L-N-B distr., with GLIMMIX';
class subject;
model y/n = / s dist=binomial link=logit ;
random int / sub=subject;
estimate 'linear pred.' int 1 / ilink cl;
run;

data williams;	*---Williams 1975 treated group, toxicological data;
input y n;
subject=_n_;
datalines;
 12 12
 11 11
 10 10
  9  9
 10 11
  9 10
  9 10
  8  9
  8  9
  4  5
  7  9
  4  7
  5 10
  3  6
  3 10
  0  7
;

title 'Fitting distrete distributions to Williams (1975) treatment group (Ex. 6)';
proc nlmixed data=williams;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
    parameters lp = 2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=williams;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
    parameters lp = -2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
								
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--Following lines give the log-linear formulation for the BBD log-likelihood;
	*--The combinatorial and the Beta functions are all re-expresed as lgamma functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  	*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;

proc nlmixed data=williams ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'L-N-B distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 gam = 0.1 ; 	*--LOGIT linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    p=1/(1+exp(-eta));					*--Get p after first getting the logit;
    model y ~ binomial(n,p);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1/(1+exp(-lp));

	ipi=1-pi;
	twopi=2*pi;
	num = pi*ipi*(1+((1-twopi)**2)*(gam*gam)/2)*(gam*gam);
	den = 1 + (gam*gam)/2 - twopi*ipi*(gam*gam) + (pi*ipi*((1-twopi)**2)/2)*(gam**4);
	rho_4 = num/den;
	
	estimate 'gam^2' gam*gam;
	estimate 'pi' 1/(1+exp(-lp));
	estimate 'rho_' (gam*gam) / ( (gam*gam) + ((3.1415**2)/3) );
	estimate 'rho_2' (gam*gam)*pi*(1-pi);
	estimate 'rho_3' ((gam*gam)*(pi*(1-pi))**2)/( (gam*gam)*(pi*(1-pi))**2 + pi*(1-pi)) ;
	estimate 'rho_4' rho_4;
run;
*^--more choices for rho (all approximations). Reminder, rho_ (no number suffix) is 
	based on different assumptions than the others.;

proc glimmix data=williams method=laplace;
title2 'L-N-B distr., with GLIMMIX';
class subject;
model y/n = / s dist=binomial link=logit ;
random int / sub=subject;
estimate 'linear pred.' int 1 / ilink cl;
run;


data skellam;	*---(4) Test of data in Skellam (J. Roy. Stat. Soc. B 10:257-261 [1948]);
input y @@;
n=3;
subject=_n_;
datalines;
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
   0   0   0   0   0   0   0   0   0   0   0   0   0   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3
;
title 'Fitting distrete distributions to Skellam (1948), Ex. 4';
proc nlmixed data=skellam;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
    parameters lp = 2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
							*--Give initial guess;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=skellam;		*---Assumes that counts/SU are in y (not x);
								*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
    parameters lp = 2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
										*--Give an initial guess;
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--Following lines give the log-linear formulation for the BBD log-likelihood;
	*--The combinatorial and the Beta functions are all re-expresed as lgamma functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  	*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;


proc nlmixed data=skellam ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'L-N-B distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 gam = 0.1 ; 	*--LOGIT linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    p=1/(1+exp(-eta));					*--Get p after first getting the logit;
    model y ~ binomial(n,p);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1/(1+exp(-lp));
	
	ipi=1-pi;
	twopi=2*pi;
	num = pi*ipi*(1+((1-twopi)**2)*(gam*gam)/2)*(gam*gam);
	den = 1 + (gam*gam)/2 - twopi*ipi*(gam*gam) + (pi*ipi*((1-twopi)**2)/2)*(gam**4);
	rho_4 = num/den;
	

	estimate 'gam^2' gam*gam;
	estimate 'pi' 1/(1+exp(-lp));
	estimate 'rho_' (gam*gam) / ( (gam*gam) + ((3.1415**2)/3) );
	estimate 'rho_2' (gam*gam)*pi*(1-pi);
	estimate 'rho_3' ((gam*gam)*(pi*(1-pi))**2)/( (gam*gam)*(pi*(1-pi))**2 + pi*(1-pi)) ;
	estimate 'rho_4' rho_4;
run;

proc glimmix data=skellam method=laplace;
title2 'L-N-B distr., with GLIMMIX';
class subject;
model y/n = / s dist=binomial link=logit ;
random int / sub=subject;
estimate 'linear pred.' int 1 / ilink cl;
run;


*---Now fit CLL-N-B distribution (CLL link, mixing on the CLL link scale).
	EXPERIMENTAL.;

proc nlmixed data=SKELLAM ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'CLL-N-B distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 gam = 0.1 ; 	*--CLL linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    *p=1/(1+exp(-eta));					*--Get p (conditional) after first getting the CLL;
	p = 1 - exp(-exp(eta));
	model y ~ binomial(n,p);			*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1 - exp(-exp(lp));
	estimate 'gam^2' gam*gam;			*--CLL variance;
	estimate 'pi' 1 - exp(-exp(lp));	*--marginal p;
	
	estimate 'rho_2' (gam*gam)*(1-pi)*(1-pi)*(-log(1-pi))*(-log(1-pi))/(pi*(1-pi));

run;
*---rho_2 is based on Taylor series expansion. Less elegant than with LNB (terms do not cancel).
	See Kriss article (cited above) for a different (maybe better?) formula for rho.;

proc glimmix data=SKELLAM method=laplace;
title2 'CLL-N-B distr., with GLIMMIX';
class subject;
model y/n = / s dist=binomial link=cll ;
random int / sub=subject;
estimate 'linear pred.' int 1 / ilink cl;
run;

data eut;
input y @@;		*<--shows that the observations are strung out across each record;
subject=_n_;	*<--need a SUBJECT for the LNB (below), each observation is a subject;
n=9;			*<--there are n=9 observations in each sampling unit;
datalines;
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
   0   0   0   0   0   0   0   0   0   0   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   4   4   4   4
   4   4   4   4   4   4   4   4   4   4   4   4   4   4   4   4   4   4   4
   4   4   4   4   4   4   4   4   4   4   5   5   5   5   5   5   5   5   5
   5   5   5   5   5   5   5   5   5   5   5   5   6   6   6   6   6   6   6
   6   6   6   6   6   7   7   7   8
;
title 'Eutypa example from Gary M. (Carneros 1990?)';
proc nlmixed data=eut;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
    parameters lp = 2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
							*--Give initial guess;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=eut;		*---Assumes that counts/SU are in y (not x);
								*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
    parameters lp = 2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
										*--Give an initial guess;
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--Following lines give the log-linear formulation for the BBD log-likelihood;
	*--The combinatorial and the Beta functions are all re-expresed as lgamma functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  	*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;


proc nlmixed data=eut ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'L-N-B distribution';
										*--Give initial guess of parameters;
    parms lp = -2.0 gam = 0.1 ; 	*--LOGIT linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    p=1/(1+exp(-eta));					*--Get p after first getting the logit;
    model y ~ binomial(n,p);			*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1/(1+exp(-lp));

	ipi=1-pi;
	twopi=2*pi;
	num = pi*ipi*(1+((1-twopi)**2)*(gam*gam)/2)*(gam*gam);
	den = 1 + (gam*gam)/2 - twopi*ipi*(gam*gam) + (pi*ipi*((1-twopi)**2)/2)*(gam**4);
	rho_4 = num/den;
	
	estimate 'gam^2' gam*gam;
	estimate 'pi' 1/(1+exp(-lp));
	estimate 'rho_' (gam*gam) / ( (gam*gam) + ((3.1415**2)/3) );
	estimate 'rho_2' (gam*gam)*pi*(1-pi);
	estimate 'rho_3' ((gam*gam)*(pi*(1-pi))**2)/( (gam*gam)*(pi*(1-pi))**2 + pi*(1-pi)) ;
	estimate 'rho_4' rho_4;
	*^--choices for the intra-cluster correlations (demo);
run;

proc glimmix data=eut method=laplace;
title2 'L-N-B distr., with GLIMMIX';
class subject;
model y/n = / s dist=binomial link=logit ;
random int / sub=subject;
estimate 'linear pred.' int 1 / ilink cl;
run;


proc nlmixed data=eut ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'CLL-N-B distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 gam = 0.1 ; 	*--CLL linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    *p=1/(1+exp(-eta));					*--Get p after first getting the CLL;
	p = 1 - exp(-exp(eta));
	model y ~ binomial(n,p);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1 - exp(-exp(lp));
	estimate 'gam^2' gam*gam;
	estimate 'pi' 1 - exp(-exp(lp));
	
	estimate 'rho_2' (gam*gam)*(1-pi)*(1-pi)*(-log(1-pi))*(-log(1-pi))/(pi*(1-pi));
	
run;

proc glimmix data=eut method=laplace;
title2 'CLL-N-B distr., with GLIMMIX';
class subject;
model y/n = / s dist=binomial link=cll ;
random int / sub=subject;
estimate 'linear pred.' int 1 / ilink cl;
run;


/* next example uses the REPLICATE statement to read frequencies into NLMIXED */

data samita;	*<--frequencies are in the file. That is, there are 36 0s, 48 1s, and so on.;
input y count;
n = 9;			*<--there are 9 observations in each sampling unit;
subject=_n_;	*<--need to subject when fitting the LNB, each observation is a subject;
datalines;
0 36
1 48
2 38
3 23
4 10
5 3
6 1
7 1
8 0
9 0
;

title 'Fitting distrete distributions to Samita & Hughes (1995), page 22';
proc nlmixed data=samita;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
	replicate count;
    parameters lp = 2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
							*--Give initial guess;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=samita;		*---Assumes that counts/SU are in y (not x);
								*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
	replicate count;
    parameters lp = 2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
										*--Give an initial guess;
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--Following lines give the log-linear formulation for the BBD log-likelihood;
	*--The combinatorial and the Beta functions are all re-expresed as lgamma functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  	*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;

/* when REPLICATE is used with RANDOM statement, a different df calculation is done. */
proc nlmixed data=samita ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'L-N-B distribution';
	replicate count;
										*--Give initial guess of parameters;
    parameters lp = -2.0 gam = 0.1 ; 	*--LOGIT linear predictor (lp) and gamma (gam);
	eta = lp + u;
	random u ~ normal(0,gam**2) sub=subject;
    p=1/(1+exp(-eta));					*--Get p after first getting the logit;
    model y ~ binomial(n,p);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	pi = 1/(1+exp(-lp));
	
	ipi=1-pi;
	twopi=2*pi;
	num = pi*ipi*(1+((1-twopi)**2)*(gam*gam)/2)*(gam*gam);
	den = 1 + (gam*gam)/2 - twopi*ipi*(gam*gam) + (pi*ipi*((1-twopi)**2)/2)*(gam**4);
	rho_4 = num/den;
	

	estimate 'gam^2' gam*gam;
	estimate 'pi' 1/(1+exp(-lp));
	estimate 'rho_' (gam*gam) / ( (gam*gam) + ((3.1415**2)/3) );
	estimate 'rho_2' (gam*gam)*pi*(1-pi);
	estimate 'rho_3' ((gam*gam)*(pi*(1-pi))**2)/( (gam*gam)*(pi*(1-pi))**2 + pi*(1-pi)) ;
	estimate 'rho_4' rho_4;
run;

******************************************************************************************;
